package com.usky.iot.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.usky.common.core.bean.CommonPage;
import com.usky.common.core.exception.BusinessException;
import com.usky.common.mybatis.core.AbstractCrudService;
import com.usky.common.security.utils.SecurityUtils;
import com.usky.iot.domain.PmProject;
import com.usky.iot.domain.PmReceive;
import com.usky.iot.domain.PmWorkContent;
import com.usky.iot.domain.PmWorkReport;
import com.usky.iot.mapper.*;
import com.usky.iot.service.PmProjectService;
import com.usky.iot.service.PmWorkContentService;
import com.usky.iot.service.vo.*;
import com.usky.system.domain.SysUser;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 工作内容表 服务实现类
 * </p>
 *
 * @author fu
 * @since 2024-05-20
 */
@Service
public class PmWorkContentServiceImpl extends AbstractCrudService<PmWorkContentMapper, PmWorkContent> implements PmWorkContentService {

    @Autowired
    private PmWorkReportMapper pmWorkReportMapper;

    @Autowired
    private SysUserMapper sysUserMapper;

    @Autowired
    private PmReceiveMapper pmReceiveMapper;

    @Autowired
    private PmWorkContentMapper pmWorkContentMapper;

    @Autowired
    private PmProjectMapper pmProjectMapper;

    @Override
    public void deleteContent(Integer reportId) {
        LambdaQueryWrapper<PmWorkContent> deleteWrapper = Wrappers.lambdaQuery();
        deleteWrapper.eq(PmWorkContent::getReportId, reportId);
        baseMapper.delete(deleteWrapper);
    }

    /**
     * @description: 查询报告id时间段
     * @author: fu
     * @date: 2024/8/7 17:48
     * @param: [startDate, endDate]
     * @return: java.util.List<java.lang.Integer>
     **/
    private List<Integer> reportDateQuery(String startDate, String endDate) {
        Long deptId = SecurityUtils.getLoginUser().getSysUser().getDeptId();
        Integer tenantId = SecurityUtils.getTenantId();
        LambdaQueryWrapper<PmWorkReport> reportQuery = Wrappers.lambdaQuery();
        reportQuery.select(PmWorkReport::getId)
                .eq(PmWorkReport::getTenantId, tenantId)
                .eq(PmWorkReport::getDeptId, deptId);
        if (StringUtils.isNotBlank(startDate) && StringUtils.isNotBlank(endDate)) {
            LocalDate start = null;
            LocalDate end = null;
            try {
                start = LocalDate.parse(startDate);
                end = LocalDate.parse(endDate);
            } catch (Exception e) {
                log.error("时间格式错误：", e);
                throw new BusinessException("时间格有误，请重新输入");
            }
            reportQuery.between(PmWorkReport::getReportDate, start, end);
        }
        List<PmWorkReport> reportIds = pmWorkReportMapper.selectList(reportQuery);
        List<Integer> rIds = new ArrayList<>();
        for (PmWorkReport report : reportIds) {
            rIds.add(report.getId());
        }
        return rIds;
    }

    /**
     * @description: 查询我收到的报告
     * @author: fu
     * @date: 2024/8/7 17:48
     * @param: []
     * @return: java.util.List<java.lang.Integer>
     **/
    private List<Integer> receive() {
        Long userId = SecurityUtils.getUserId();
        Integer tenantId = SecurityUtils.getTenantId();
        LambdaQueryWrapper<PmWorkReport> reportQuery = Wrappers.lambdaQuery();
        reportQuery.select(PmWorkReport::getId)
                .eq(PmWorkReport::getTenantId, tenantId)
                .apply("FIND_IN_SET(" + userId + ", cc_to) > 0");
        List<PmWorkReport> reportIds = pmWorkReportMapper.selectList(reportQuery);
        return reportIds.stream().map(PmWorkReport::getId).collect(Collectors.toList());
    }

    /**
     * @description: 查询我发出的报告
     * @author: fu
     * @date: 2024/8/7 17:48
     * @param: []
     * @return: java.util.List<java.lang.Integer>
     **/
    private List<Integer> sentOut() {
        Long userid = SecurityUtils.getUserId();
        Integer tenantId = SecurityUtils.getTenantId();
        LambdaQueryWrapper<PmWorkReport> reportQuery = Wrappers.lambdaQuery();
        reportQuery.select(PmWorkReport::getId)
                .eq(PmWorkReport::getTenantId, tenantId)
                .eq(PmWorkReport::getSubmitterId, userid);
        List<PmWorkReport> reportIds = pmWorkReportMapper.selectList(reportQuery);
        List<Integer> rIds = new ArrayList<>();
        for (PmWorkReport report : reportIds) {
            rIds.add(report.getId());
        }
        return rIds;
    }

    /**
     * @description: 负责人项目id查询
     * @author: fu
     * @date: 2024/8/7 17:14
     * @param: []
     * @return: java.util.List<java.lang.Integer>
     **/
    private List<Integer> head() {
        Long userId = SecurityUtils.getUserId();
        Integer tenantId = SecurityUtils.getTenantId();
        LambdaQueryWrapper<PmProject> projectQuery = Wrappers.lambdaQuery();
        projectQuery.select(PmProject::getId)
                .eq(PmProject::getTenantId, tenantId)
                .eq(PmProject::getProjectHead, userId);
        List<PmProject> projectIds = pmProjectMapper.selectList(projectQuery);
        List<Integer> rIds = new ArrayList<>();
        for (PmProject project : projectIds) {
            rIds.add(project.getId());
        }
        return rIds;
    }

    /**
     * @description: 查询项目id
     * @author: fu
     * @date: 2024/8/7 17:14
     * @param: []
     * @return: java.util.List<java.lang.Integer>
     **/
    private List<Integer> reportList(String projectName) {
        Integer tenantId = SecurityUtils.getTenantId();
        LambdaQueryWrapper<PmWorkContent> projectQuery = Wrappers.lambdaQuery();
        projectQuery.select(PmWorkContent::getReportId)
                .eq(PmWorkContent::getProjectName, projectName)
                .eq(PmWorkContent::getTenantId, tenantId);
        List<PmWorkContent> projectIds = pmWorkContentMapper.selectList(projectQuery);
        List<Integer> rIds = new ArrayList<>();
        for (PmWorkContent contenReportId : projectIds) {
            rIds.add(contenReportId.getId());
        }
        return rIds;
    }

    /**
     * @description: 查询项目
     * @author: fu
     * @date: 2024/8/7 17:14
     * @param: [projectIds]
     * @return: java.util.List<com.usky.iot.domain.PmProject>
     **/
    @Override
    public List<PmWorkContent> projectQuery(String startDate, String endDate, String projectName, Integer projectAscription) {
        LocalDateTime startDateTime = null;
        LocalDateTime endDateTime = null;
        LocalDate start = null;
        LocalDate end = null;
        List<Integer> reportIds = new ArrayList<>();
        if (StringUtils.isNotBlank(startDate) && StringUtils.isNotBlank(endDate)) {
            DateTimeFormatter formatter = null;
            try {
                formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
            } catch (Exception e) {
                throw new BusinessException("时间传参格式不正确请重试");
            }
            start = LocalDate.parse(startDate, formatter);
            end = LocalDate.parse(endDate, formatter);
            // startDateTime = start.atStartOfDay();
            // endDateTime = end.atTime(23, 59, 59);
            LambdaQueryWrapper<PmWorkReport> reportQuery = Wrappers.lambdaQuery();
            reportQuery.select(PmWorkReport::getId)
                    .between(PmWorkReport::getReportDate, start, end);
            reportIds = pmWorkReportMapper.selectList(reportQuery).stream().map(PmWorkReport::getId).collect(Collectors.toList());
        }

        List<PmWorkContent> workContentList = new ArrayList<>();

        LambdaQueryWrapper<PmWorkContent> contentLambdaQuery = Wrappers.lambdaQuery();
        contentLambdaQuery.select(PmWorkContent::getProjectId, PmWorkContent::getProjectName);
        switch (projectAscription) {
            case 1:
                List<Integer> headProjects = head();
                if (headProjects.isEmpty()) {
                    return workContentList;
                }
                contentLambdaQuery.in(PmWorkContent::getProjectId, headProjects);
                break;
            case 2:
                List<Integer> receiveList = receive();
                if (receiveList.isEmpty()) {
                    return workContentList;
                }
                contentLambdaQuery.in(PmWorkContent::getReportId, receiveList);
                break;
            case 3:
                List<Integer> sentOutList = sentOut();
                if (sentOutList.isEmpty()) {
                    return workContentList;
                }
                contentLambdaQuery.in(PmWorkContent::getReportId, sentOutList);
                break;
            default:
                throw new BusinessException("查询标识有误");
        }
        if (StringUtils.isNotBlank(projectName)) {
            contentLambdaQuery.like(PmWorkContent::getProjectName, projectName);
        }
        if (!reportIds.isEmpty()) {
            contentLambdaQuery.in(PmWorkContent::getReportId, reportIds);
        }
        contentLambdaQuery.groupBy(PmWorkContent::getProjectId)
                .groupBy(PmWorkContent::getProjectName);
        workContentList = pmWorkContentMapper.selectList(contentLambdaQuery);

        return workContentList;
    }

    /**
     * @description: 报告记录
     * @author: fu
     * @date: 2024/8/7 17:14
     * @param: [projectIds]
     * @return: java.util.List<com.usky.iot.domain.PmProject>
     **/
    @Override
    public List<PmWorkReport> workReportQuery(String startDate, String endDate, String projectName, Integer projectAscription, Integer reportId) {

        LocalDate start = null;
        LocalDate end = null;
        LocalDateTime startDateTime = null;
        LocalDateTime endDateTime = null;
        if (StringUtils.isNotBlank(startDate) && StringUtils.isNotBlank(endDate)) {
            DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
            start = LocalDate.parse(startDate, formatter);
            end = LocalDate.parse(endDate, formatter);
            startDateTime = start.atStartOfDay();
            endDateTime = end.atTime(23, 59, 59);
        }

        List<PmWorkReport> reportList = new ArrayList<>();
        List<PmWorkReport> reportList2 = new ArrayList<>();
        List<PmWorkContent> pmWorkContentList = new ArrayList<>();
        List<Long> userIds = new ArrayList<>();
        if (reportId != null && reportId != 0) {
            LambdaQueryWrapper<PmWorkReport> reportQuery = Wrappers.lambdaQuery();
            reportQuery.select(PmWorkReport::getId, PmWorkReport::getCoordinateWork, PmWorkReport::getTomorrowPlan, PmWorkReport::getCcTo, PmWorkReport::getSubmitterId,
                            PmWorkReport::getCreateTime, PmWorkReport::getReportDate)
                    .eq(PmWorkReport::getId, reportId);
            PmWorkReport report = pmWorkReportMapper.selectOne(reportQuery);
            if (report == null) {
                throw new BusinessException("报告已经不存在了哟");
            }
            LambdaQueryWrapper<PmWorkContent> contentQuery = Wrappers.lambdaQuery();
            contentQuery.select(PmWorkContent::getWorkContent, PmWorkContent::getWorkTime, PmWorkContent::getProjectName)
                    .eq(PmWorkContent::getReportId, reportId);
            pmWorkContentList = this.list(contentQuery);
            report.setWorkContents(pmWorkContentList);
            reportList.add(report);
            for (PmWorkReport c : reportList) {
                userIds.add(c.getSubmitterId());
            }
        } else {
            if (projectAscription != 1 && projectAscription != 2 && projectAscription != 3) {
                throw new BusinessException("查询标识有误");
            }
            List<Integer> receiveList2 = receive();
            if (projectAscription == 2 && receiveList2.isEmpty()) {
                return reportList2;
            }
            List<Integer> sentOutList = sentOut();
            if (projectAscription == 3 && sentOutList.isEmpty()) {
                return reportList2;
            }
            List<Integer> headProjects = head();
            if (projectAscription == 1 && headProjects.isEmpty()) {
                return reportList2;
            }
            LambdaQueryWrapper<PmWorkContent> contentLambdaQuery = Wrappers.lambdaQuery();
            contentLambdaQuery.select(PmWorkContent::getReportId, PmWorkContent::getWorkContent, PmWorkContent::getWorkTime, PmWorkContent::getProjectName);

            if (startDateTime != null) {
                contentLambdaQuery.between(PmWorkContent::getCreateTime, startDateTime, endDateTime);
            }
            if (projectAscription == 1) {
                contentLambdaQuery.in(PmWorkContent::getProjectId, headProjects);
            } else if (projectAscription == 2) {
                contentLambdaQuery.in(PmWorkContent::getReportId, receiveList2);
            } else {
                contentLambdaQuery.in(PmWorkContent::getReportId, sentOutList);
            }
            if (StringUtils.isNotBlank(projectName)) {
                List<Integer> list = reportList(projectName);
                contentLambdaQuery.in(PmWorkContent::getReportId, list);
            }
            pmWorkContentList = this.list(contentLambdaQuery);
            if (pmWorkContentList.isEmpty()) {
                return reportList;
            }
            List<Integer> reportIds = new ArrayList<>();
            for (PmWorkContent a : pmWorkContentList) {
                reportIds.add(a.getReportId());
            }
            LambdaQueryWrapper<PmWorkReport> reportQuery = Wrappers.lambdaQuery();
            reportQuery.select(PmWorkReport::getId, PmWorkReport::getCoordinateWork, PmWorkReport::getTomorrowPlan, PmWorkReport::getCcTo, PmWorkReport::getSubmitterId,
                            PmWorkReport::getCreateTime, PmWorkReport::getReportDate)
                    .in(PmWorkReport::getId, reportIds)
                    .orderByDesc(PmWorkReport::getCreateTime);
            reportList = pmWorkReportMapper.selectList(reportQuery);
            LambdaQueryWrapper<PmReceive> statusQuery = Wrappers.lambdaQuery();
            statusQuery.select(PmReceive::getReportId, PmReceive::getReadFlag)
                    .eq(PmReceive::getReceiverId, SecurityUtils.getUserId())
                    .eq(PmReceive::getTenantId, SecurityUtils.getTenantId())
                    .in(PmReceive::getReportId, reportIds);
            List<PmReceive> receiveList = pmReceiveMapper.selectList(statusQuery);
            for (PmWorkReport c : reportList) {
                List<PmWorkContent> contents = new ArrayList<>();
                for (PmWorkContent b : pmWorkContentList) {
                    if (b.getReportId().equals(c.getId())) {
                        contents.add(b);
                    }
                }
                c.setWorkContents(contents);
                if (!receiveList.isEmpty()) {
                    for (PmReceive receive : receiveList) {
                        if (receive.getReportId().equals(c.getId())) {
                            c.setReadFlag(receive.getReadFlag());
                        }
                    }
                }
                userIds.add(c.getSubmitterId());
            }
        }
        List<SysUser> nickNames = nickNames(userIds);
        for (PmWorkReport d : reportList) {
            for (SysUser e : nickNames) {
                if (e.getUserId().equals(d.getSubmitterId())) {
                    d.setCreateBy(e.getNickName());
                }
            }
            List<Long> ids = null;
            if (!d.getCcTo().isEmpty()) {
                ids = Optional.ofNullable(d.getCcTo())
                        .map(ccTo -> Arrays.stream(ccTo.split(","))
                                .map(Long::parseLong)
                                .collect(Collectors.toList()))
                        .orElse(Collections.emptyList());
            }
            List<SysUser> nickNameList = null;
            if (CollectionUtils.isNotEmpty(ids)) {
                nickNameList = nickNames(ids);
                StringBuilder ccToBuilder = new StringBuilder();
                Set<Long> addedUserIds = new HashSet<>();
                for (Long id : ids) {
                    for (SysUser user : nickNameList) {
                        if (user.getUserId().equals(id) && !addedUserIds.contains(id)) {
                            if (ccToBuilder.length() > 0) {
                                ccToBuilder.append(", ");
                            }
                            ccToBuilder.append(user.getNickName());
                            addedUserIds.add(id);
                            break;
                        }
                    }
                }
                String ccTo = ccToBuilder.toString();
                d.setCcTo(ccTo);
            }
        }
        return reportList;
    }

    /**
     * @description: 用户名、账户名查询
     * @author: fu
     * @date: 2024/8/7 17:28
     * @param: [userIds]
     * @return: java.util.List<com.usky.system.domain.SysUser>
     **/
    @Override
    public List<SysUser> nickNames(List<Long> userIds) {
        LambdaQueryWrapper<SysUser> usersQuery = Wrappers.lambdaQuery();
        usersQuery.select(SysUser::getUserId, SysUser::getNickName, SysUser::getUserName, SysUser::getPhonenumber, SysUser::getAvatar,
                        SysUser::getSex, SysUser::getDeptId)
                .eq(SysUser::getTenantId, SecurityUtils.getTenantId())
                .eq(SysUser::getDelFlag, 0);
        if (!userIds.isEmpty()) {
            usersQuery.in(SysUser::getUserId, userIds);
        }
        return sysUserMapper.selectList(usersQuery);
    }

    @Override
    public List<SysUser> allNickName() {
        LambdaQueryWrapper<SysUser> usersQuery = Wrappers.lambdaQuery();
        usersQuery.select(SysUser::getUserId, SysUser::getNickName, SysUser::getUserName)
                .eq(SysUser::getTenantId, SecurityUtils.getTenantId())
                .eq(SysUser::getDelFlag, 0);
        return sysUserMapper.selectList(usersQuery);
    }

    /**
     * @description: 报告记录分页
     * @author: fu
     * @date: 2024/8/7 17:28
     * @param: [startDate, endDate, projectName, projectAscription]
     * @return: java.util.List<com.usky.system.domain.PmWorkContent>
     **/
    @Override
    public CommonPage<PmWorkReport> reportPage(Integer projectAscription, Integer pageNum, Integer pageSize, Integer reportId, String startDate, String endDate, Integer projectId) {
        LocalDateTime startDateTime = null;
        LocalDateTime endDateTime = null;
        LocalDate start = null;
        LocalDate end = null;
        List<Integer> reportIds = new ArrayList<>();
        if (StringUtils.isNotBlank(startDate) && StringUtils.isNotBlank(endDate)) {
            DateTimeFormatter formatter = null;
            try {
                formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
            } catch (Exception e) {
                throw new BusinessException("时间传参格式不正确请重试");
            }
            start = LocalDate.parse(startDate, formatter);
            end = LocalDate.parse(endDate, formatter);
            // startDateTime = start.atStartOfDay();
            // endDateTime = end.atTime(23, 59, 59);
            LambdaQueryWrapper<PmWorkReport> reportQuery = Wrappers.lambdaQuery();
            reportQuery.select(PmWorkReport::getId)
                    .between(PmWorkReport::getReportDate, start, end);
            reportIds = pmWorkReportMapper.selectList(reportQuery).stream().map(PmWorkReport::getId).collect(Collectors.toList());
        }

        if (projectId != null && projectId > 0) {
            LambdaQueryWrapper<PmWorkContent> contentLambdaQuery = Wrappers.lambdaQuery();
            contentLambdaQuery.select(PmWorkContent::getReportId)
                    .eq(PmWorkContent::getProjectId, projectId);
            reportIds.addAll(pmWorkContentMapper.selectList(contentLambdaQuery).stream().map(PmWorkContent::getReportId).collect(Collectors.toList()));
        }

        CommonPage<PmWorkReport> returnPage = new CommonPage<>(new ArrayList<>(), 0, pageSize, pageNum);
        Integer tenantId = SecurityUtils.getTenantId();
        Long userId = SecurityUtils.getUserId();
        List<Long> userIds = new ArrayList<>();
        List<Integer> receiveList = receive();
        if (receiveList.isEmpty()) {
            return returnPage;
        }

        LambdaQueryWrapper<PmWorkContent> contentLambdaQuery = Wrappers.lambdaQuery();
        contentLambdaQuery.select(PmWorkContent::getReportId, PmWorkContent::getWorkContent, PmWorkContent::getWorkTime, PmWorkContent::getProjectName);
        if (reportId != null && reportId > 0) {
            contentLambdaQuery.eq(PmWorkContent::getReportId, reportId);
        } else {
            switch (projectAscription) {
                case 1:
                    List<Integer> headProjects = head();
                    if (headProjects.isEmpty()) {
                        return returnPage;
                    }
                    contentLambdaQuery.in(PmWorkContent::getProjectId, headProjects);
                    break;
                case 2:
                    contentLambdaQuery.in(PmWorkContent::getReportId, receiveList);
                    break;
                case 3:
                    List<Integer> sentOutList = sentOut();
                    if (sentOutList.isEmpty()) {
                        return returnPage;
                    }
                    contentLambdaQuery.in(PmWorkContent::getReportId, sentOutList);
                    break;
                default:
                    throw new BusinessException("查询标识有误!");
            }
            if (!reportIds.isEmpty()) {
                contentLambdaQuery.in(PmWorkContent::getReportId, reportIds);
            }
        }
        List<PmWorkContent> pmWorkContentList = pmWorkContentMapper.selectList(contentLambdaQuery);
        if (pmWorkContentList.isEmpty()) {
            if (reportId != null && reportId > 0) {
                throw new BusinessException("该报告已被删除！");
            }
            return returnPage;
        }
        List<Integer> reportIds2 = pmWorkContentList.stream().distinct().map(PmWorkContent::getReportId).collect(Collectors.toList());

        LambdaQueryWrapper<PmWorkReport> reportQuery = Wrappers.lambdaQuery();
        reportQuery.select(PmWorkReport::getId, PmWorkReport::getCoordinateWork, PmWorkReport::getTomorrowPlan, PmWorkReport::getCcTo, PmWorkReport::getSubmitterId,
                        PmWorkReport::getSubmitDate, PmWorkReport::getReportDate, PmWorkReport::getCreateBy)
                .eq(PmWorkReport::getReportStatus, 1);
        if (reportId != null && reportId != 0) {
            reportQuery.eq(PmWorkReport::getId, reportId);
        } else {
            reportQuery.in(PmWorkReport::getId, reportIds2)
                    .orderByDesc(PmWorkReport::getCreateTime);
        }
        IPage<PmWorkReport> reportPage = pmWorkReportMapper.selectPage(new Page<>(pageNum, pageSize), reportQuery);
        List<PmWorkReport> reportList = reportPage.getRecords();

        // 查询已读状态
        LambdaQueryWrapper<PmReceive> statusQuery = Wrappers.lambdaQuery();
        statusQuery.select(PmReceive::getReportId, PmReceive::getReadFlag)
                .eq(PmReceive::getReceiverId, userId);
        if (reportId != null && reportId != 0) {
            statusQuery.eq(PmReceive::getReportId, reportId);
        } else if (!receiveList.isEmpty()) {
            statusQuery.in(PmReceive::getReportId, receiveList);
        }
        List<PmReceive> receiveList2 = pmReceiveMapper.selectList(statusQuery);

        Map<Integer, Integer> reportReadFlags = new HashMap<>();
        for (PmReceive pmReceive : receiveList2) {
            Integer reportId2 = pmReceive.getReportId();
            reportReadFlags.put(reportId2, pmReceive.getReadFlag());
        }

        for (PmWorkReport report : reportList) {
            List<PmWorkContent> contents = pmWorkContentList.stream()
                    .filter(content -> content.getReportId().equals(report.getId()))
                    .collect(Collectors.toList());
            report.setWorkContents(contents);
            userIds.add(report.getSubmitterId());

            Integer readFlagValue = reportReadFlags.get(report.getId());
            report.setReadFlag(readFlagValue);

            // 创建人名字替换
            if (!userIds.isEmpty()) {
                List<SysUser> nickNames = nickNames(userIds);
                Map<Long, String> userNicknameMap = nickNames.stream()
                        .collect(Collectors.toMap(SysUser::getUserId, SysUser::getNickName));
                report.setCreateBy(userNicknameMap.get(report.getSubmitterId()));
            }
        }


        // 已读未读数量查询
        List<PmReceive> receives = receives(receiveList);
        List<Long> uId = new ArrayList<>();
        for (PmReceive pmReceive : receives) {
            Long receiverId = pmReceive.getReceiverId();
            uId.add(receiverId);
        }


        Set<Long> userIdSet = new HashSet<>();
        for (PmWorkReport report : reportList) {
            Long submitterId = report.getSubmitterId();
            userIdSet.add(submitterId);
        }
        List<Long> userIdList = new ArrayList<>(userIdSet);

        List<SysUser> sysUsers = nickNames(userIdList);

        Map<Integer, List<PmReceive>> reportReceivesMap = receives.stream()
                .collect(Collectors.groupingBy(PmReceive::getReportId));

        if (sysUsers != null) {
            for (PmWorkReport report : reportList) {
                List<Long> readAlready = new ArrayList<>();
                List<Long> readNotAlready = new ArrayList<>();
                int readCount = 0;
                int unreadCount = 0;
                List<PmReceive> reportReceives = reportReceivesMap.getOrDefault(report.getId(), Collections.emptyList());
                for (PmReceive pmReceive : reportReceives) {
                    if (pmReceive.getReadFlag() == 1) {
                        readCount++;
                        readAlready.add(pmReceive.getReceiverId());
                    } else {
                        unreadCount++;
                        readNotAlready.add(pmReceive.getReceiverId());
                    }
                }

                PmReportReadersVO readUnreadVO = new PmReportReadersVO(readAlready, readNotAlready, readCount, unreadCount);
                report.setPmReportReaders(readUnreadVO);

                // 头像
                for (SysUser sysUser : sysUsers) {
                    if (sysUser.getUserId().equals(report.getSubmitterId())) {
                        report.setAvatar(sysUser.getAvatar());
                        break;
                    }
                }
            }
        }

        return new CommonPage<>(reportList, reportPage.getTotal(), pageSize, pageNum);
    }

    /**
     * @description: 获取报告接收人
     * @author: fu
     * @date: 2024/8/8 19:22
     * @param: [report]
     * @return: java.util.List<com.usky.iot.domain.PmReceive>
     **/
    private List<PmReceive> receives(List<Integer> reports) {
        LambdaQueryWrapper<PmReceive> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.select(PmReceive::getReportId, PmReceive::getReadFlag, PmReceive::getReceiverId)
                .in(PmReceive::getReportId, reports);
        return pmReceiveMapper.selectList(queryWrapper);
    }

    /**
     * @description: 获取报告列表
     * @author: fu
     * @date: 2024/8/8 19:22
     * @param: [startDate, endDate]
     * @return: java.util.List<java.lang.Integer>
     **/
    private List<Integer> reports(LocalDate startDate, LocalDate endDate) {
        LambdaQueryWrapper<PmWorkReport> reportsQuery = Wrappers.lambdaQuery();
        reportsQuery.select(PmWorkReport::getId)
                .between(PmWorkReport::getReportDate, startDate, endDate)
                .groupBy(PmWorkReport::getId);
        List<PmWorkReport> pmWorkReportList = pmWorkReportMapper.selectList(reportsQuery);
        return pmWorkReportList.stream()
                .map(PmWorkReport::getId)
                .collect(Collectors.toList());
    }

    private List<PmProject> projects(Integer projectId) {
        Integer tenantId = SecurityUtils.getTenantId();
        Long deptId = SecurityUtils.getLoginUser().getSysUser().getDeptId();
        LambdaQueryWrapper<PmProject> projectsQuery = Wrappers.lambdaQuery();
        projectsQuery.select(PmProject::getId, PmProject::getProjectName);
        if (projectId != null && projectId != 0) {
            projectsQuery.eq(PmProject::getId, projectId);
            return pmProjectMapper.selectList(projectsQuery);
        }
        projectsQuery.eq(PmProject::getTenantId, tenantId).eq(PmProject::getDeptId, deptId).eq(PmProject::getDelFlag, 0);
        return pmProjectMapper.selectList(projectsQuery);
    }

    private List<SysUser> userNameList(Long userId) {
        Integer tenantId = SecurityUtils.getTenantId();
        Long deptId = SecurityUtils.getLoginUser().getSysUser().getDeptId();
        LambdaQueryWrapper<SysUser> userNameQuery = Wrappers.lambdaQuery();
        userNameQuery.select(SysUser::getUserId, SysUser::getNickName, SysUser::getUserName, SysUser::getPhonenumber, SysUser::getAvatar,
                        SysUser::getSex, SysUser::getDeptId)
                .eq(SysUser::getDelFlag, 0)
                .eq(SysUser::getStatus, 0);
        if (userId != null && userId != 0) {
            userNameQuery.eq(SysUser::getUserId, userId);
            return sysUserMapper.selectList(userNameQuery);
        }
        userNameQuery.eq(SysUser::getTenantId, tenantId)
                .eq(SysUser::getDeptId, deptId);
        return sysUserMapper.selectList(userNameQuery);
    }

    //@DataScope//数据权限注解
    private List<PmWorkContent> workTimeCount(List<Long> users, List<Integer> project, String startDate, String endDate) {
        Integer tenantId = SecurityUtils.getTenantId();
        Long deptId = SecurityUtils.getLoginUser().getSysUser().getDeptId();
        LambdaQueryWrapper<PmWorkContent> workTimeQuery = Wrappers.lambdaQuery();
        workTimeQuery.select(PmWorkContent::getSubmitterId, PmWorkContent::getReportId, PmWorkContent::getProjectId, PmWorkContent::getProjectName,
                PmWorkContent::getWorkTime, PmWorkContent::getDeptId, PmWorkContent::getTenantId);
        if (!users.isEmpty()) {
            workTimeQuery.in(PmWorkContent::getSubmitterId, users);
        }
        if (!project.isEmpty()) {
            workTimeQuery.in(PmWorkContent::getProjectId, project);
        }
        if (StringUtils.isNotBlank(startDate) && StringUtils.isNotBlank(endDate)) {
            LocalDate start = LocalDate.parse(startDate);
            LocalDate end = LocalDate.parse(endDate);
            List<Integer> reports = reports(start, end);
            workTimeQuery.in(PmWorkContent::getReportId, reports);
        }
        workTimeQuery.eq(PmWorkContent::getTenantId, tenantId).eq(PmWorkContent::getDeptId, deptId);
        // workTimeQuery.apply(DataScopeContextHolder.getDataScopeSql());// 数据权限设置
        List<PmWorkContent> pmWorkContentList = pmWorkContentMapper.selectList(workTimeQuery);

        Map<String, PmWorkContent> workTimeMap = new HashMap<>();
        for (PmWorkContent content : pmWorkContentList) {
            String key = content.getSubmitterId() + "_" + content.getProjectId();
            PmWorkContent existingContent = workTimeMap.get(key);
            if (existingContent == null) {
                workTimeMap.put(key, content);
            } else {
                existingContent.setWorkTime(existingContent.getWorkTime().add(content.getWorkTime()));
            }
        }

        return new ArrayList<>(workTimeMap.values());
    }


    @Override
    public List<Object> workHourStatistic(Long userId, Integer projectId, Integer filter, String startDate, String endDate, Integer workerOrProject) {
        List<ProjectWorkTimeVO> projectWorkTime = new ArrayList<>();
        List<UserWorkTimeVO> userWorkTimeVO = new ArrayList<>();
        List<Object> returnList = new ArrayList<>();
        if (StringUtils.isBlank(startDate) && StringUtils.isBlank(endDate)) {
            throw new BusinessException("请设置查询时间范围！");
        }

        List<PmProject> pmProjects = projects(projectId);
        List<String> pmProjectName = new ArrayList<>();
        List<Integer> pmProjectId = new ArrayList<>();
        for (PmProject project : pmProjects) {
            pmProjectName.add(project.getProjectName());
            pmProjectId.add(project.getId());
        }

        List<SysUser> users = userNameList(userId);
        List<Long> userIds = new ArrayList<>();
        List<String> nickName = new ArrayList<>();
        for (SysUser user : users) {
            userIds.add(user.getUserId());
            nickName.add(user.getNickName());
        }

        // 统计userId每个项目所需工时
        List<WorkHoursStatisticsVO> userAndProject = new ArrayList<>();
        List<PmWorkContent> pmWorkContentList = workTimeCount(userIds, pmProjectId, startDate, endDate);
        for (PmWorkContent content : pmWorkContentList) {
            WorkHoursStatisticsVO workHoursStatisticsVO = new WorkHoursStatisticsVO();
            Long submitterId = content.getSubmitterId();
            Integer projectId1 = content.getProjectId();
            BigDecimal workTime = content.getWorkTime();
            workHoursStatisticsVO.setWorkTime(workTime);
            for (SysUser user : users) {
                Long userId1 = user.getUserId();
                String nickName1 = user.getNickName();
                if (userId1.equals(submitterId)) {
                    workHoursStatisticsVO.setNickName(nickName1);
                }
            }
            for (PmProject project : pmProjects) {
                Integer projectId2 = project.getId();
                String projectName = project.getProjectName();
                if (projectId2 == projectId1) {
                    workHoursStatisticsVO.setProjectName(projectName);
                }
            }
            userAndProject.add(workHoursStatisticsVO);
        }
        List<WorkHoursStatisticsVO> userAndProjectD = userAndProject;

        if (workerOrProject != 1) {
            returnList.add(nickName);
            // 重组返回数据结构
            for (String name : pmProjectName) {
                ProjectWorkTimeVO projectWorkTimeVO = new ProjectWorkTimeVO();
                projectWorkTimeVO.setProjectOrPerson(name);
                List<BigDecimal> workTime = new ArrayList<>();
                boolean hasNonZeroWorkTime = false;
                for (String currentNickName : nickName) {
                    boolean foundWorkTime = false;
                    for (WorkHoursStatisticsVO workHoursStatisticsVO : userAndProject) {
                        String userNickName = workHoursStatisticsVO.getNickName();
                        String projectName = workHoursStatisticsVO.getProjectName();
                        BigDecimal userProjectWorkTime = workHoursStatisticsVO.getWorkTime();
                        if (projectName == name && userNickName == currentNickName) {
                            workTime.add(userProjectWorkTime);
                            if (userProjectWorkTime.compareTo(BigDecimal.ZERO) != 0) {
                                hasNonZeroWorkTime = true;
                            }
                            foundWorkTime = true;
                            break;
                        }
                    }
                    if (!foundWorkTime) {
                        workTime.add(BigDecimal.ZERO);
                    }
                }
                if (filter != 2 || hasNonZeroWorkTime) {
                    projectWorkTimeVO.setWorkTime(workTime);
                    projectWorkTime.add(projectWorkTimeVO);
                }
            }
            returnList.add(projectWorkTime);
        } else {
            returnList.add(pmProjectName);
            // 重组返回数据结构
            for (String uName : nickName) {
                String name = uName;
                UserWorkTimeVO userWorkTimeVO2 = new UserWorkTimeVO();
                userWorkTimeVO2.setProjectOrPerson(name);
                List<BigDecimal> workTime = new ArrayList<>();
                boolean hasNonZeroWorkTime = false;
                for (String currentNickName : pmProjectName) {
                    boolean foundWorkTime = false;
                    for (WorkHoursStatisticsVO workHoursStatisticsVO : userAndProjectD) {
                        String userNickName = workHoursStatisticsVO.getNickName();
                        String projectName = workHoursStatisticsVO.getProjectName();
                        BigDecimal userProjectWorkTime = workHoursStatisticsVO.getWorkTime();
                        if (userNickName == name && projectName == currentNickName) {
                            workTime.add(userProjectWorkTime);
                            if (userProjectWorkTime.compareTo(BigDecimal.ZERO) != 0) {
                                hasNonZeroWorkTime = true;
                            }
                            foundWorkTime = true;
                            break;
                        }
                    }
                    if (!foundWorkTime) {
                        workTime.add(BigDecimal.ZERO);
                    }
                }
                if (filter != 2 || hasNonZeroWorkTime) {
                    userWorkTimeVO2.setWorkTime(workTime);
                    userWorkTimeVO.add(userWorkTimeVO2);
                }
            }
            returnList.add(userWorkTimeVO);
        }
        return returnList;
    }

    @Override
    public List<WorkTimeExportVO> workHourStatisticExport(Long userId, Integer projectId, String startDate, String endDate) {
        Integer tenantId = SecurityUtils.getTenantId();
        LocalDate start = null;
        LocalDate end = null;
        if (StringUtils.isNotBlank(startDate) && StringUtils.isNotBlank(endDate)) {
            start = LocalDate.parse(startDate);
            end = LocalDate.parse(endDate);
        }
        return pmWorkContentMapper.workHourStatisticExport(userId, projectId, start, end, tenantId);
    }

}
